module Tungsten
  module FormHelper
    include Tungsten::ButtonHelper

    INPUT_OPTIONS = {
      email: {
        type: "email",
        placeholder: "Email address",
        pattern: "[^@]+@[^@]+\\.[a-zA-Z]{2,}",
        autocorrect: "off",
        autocapitalize: "off",
        spellcheck: "false",
        data: { message: "Please enter a valid email address." }
      },

      password: {
        type: "password",
        placeholder: "Password"
      },

      text: {
        type: "text"
      },

      number: {
        type: "number",
        step: "any"
      },

      tel: {
        type: "tel",
        placeholder: "Phone number"
      },

      url: {
        type: "text",
        placeholder: "Web address",
        autocorrect: "off",
        autocapitalize: "off",
        spellcheck: "false",
        pattern: ".+\\.[a-zA-Z]{2,}"
      },

      card_number: {
        type: "text",
        required: true,
        pattern: "[0-9 -]{13,20}",
        placeholder: "Credit Card Number",
        data: {
          stripe: "number",
          message: "Please enter a valid credit card number."
        }
      },

      card_month: {
        type: "text",
        required: true,
        pattern: "0[1-9]|1[012]",
        placeholder: "MM",
        data: {
          stripe: "exp_month",
          message: "Please enter a valid expiration month."
        }
      },

      card_year: {
        type: "text",
        required: true,
        pattern: "[0-9]{4}",
        placeholder: "YYYY",
        data: {
          stripe: "exp_year",
          message: "Please enter a valid expiration year."
        }
      },

      card_cvc: {
        type: "text",
        required: true,
        pattern: "[0-9]{3,4}",
        placeholder: "CVC",
        data: {
          stripe: "cvc",
          message: "Please enter a valid security code."
        }
      },

      select_country: {
        type: "select",
        country_options: {
          include_blank: "Select a country",
          priority_countries: ["US", "GB", "CA"],
        },

        html_options: {}
      }
    }

    def table_form_for(record, options = {}, &block)
      form_for record, options do |f|
        table_form_tag f, &block
      end
    end

    def table_form_tag(form = nil)
      form.style = 'table' if form
      content_tag :div, class: ['table', 'table-form'] do
        yield form if block_given?
      end
    end

    def stacked_form_for(record, options = {}, &block)
      form_for record, options do |f|
        stacked_form_tag f, &block
      end
    end

    def stacked_form_tag(form = nil)
      form.style = 'stacked' if form
      content_tag :div, class: 'stacked-form' do
        yield form if block_given?
      end
    end

    # Country select
    def select_country_tag(name, options = {}, country_options = {})
      country_options.reverse_merge! INPUT_OPTIONS[:select_country][:country_options]

      options = INPUT_OPTIONS[:select_country][:html_options].deep_merge options
      options[:class] ||= ' '
      options[:class] += " #{label_class(:select)}"

      content_tag(:label, class: options.delete(:class) ) do
        country_select :user, :country, country_options, options
      end
    end

    # Email inputs
    def email_input_tag(name, value = nil, options = {})
      input_tag(:email, name, value, options)
    end

    # Passowrd inputs
    def password_input_tag(name, value = nil, options = {})
      input_tag(:password, name, value, options)
    end

    def text_input_tag(name, value = nil, options = {})
      input_tag(:text, name, value, options)
    end

    def url_input_tag(name, value = nil, options = {})
      input_tag(:url, name, value, options)
    end

    def tel_input_tag(name, value = nil, options = {})
      input_tag(:tel, name, value, options)
    end

    def textarea_tag(name, value = nil, options = {}, &block)
      input_tag(:textarea, name, value, options, &block)
    end

    def number_input_tag(name, value = nil, options = {})
      input_tag(:number, name, value, options)
    end

    def search_input_tag(name, value = nil, options = {})
      input_tag(:search, name, value, options)
    end

    def card_number_tag(name, value=nil, options={})
      input_tag(:card_number, name, value, options)
    end

    def card_month_tag(name, value=nil, options={})
      input_tag(:card_month, name, value, options)
    end

    def card_year_tag(name, value=nil, options={})
      input_tag(:card_year, name, value, options)
    end

    def card_cvc_tag(name, value=nil, options={})
      input_tag(:card_cvc, name, value, options)
    end

    def radio_button_input_tag(name, value, checked = false, options = {})

      if checked.is_a? Hash
        options = checked
        checked = false
      end

      options[:type] = :radio
      (options[:class] ||= '') << ' input'
      options[:data] ||= {}
      options[:data][:show] = options.delete(:show)

      tick_wrapper( name, options ) do
        radio_button_tag(name, value, checked, options)
      end
    end

    def slider_input_tag(name, options={})
      input_tag(:range, name, options)
    end

    def checkbox_input_tag(name, checked = false, options = {})
      value = true

      if checked.is_a? Hash
        options = checked
        checked = false
      end

      options[:type] = :checkbox

      tick_wrapper( name, options ) do
        options[:class] = 'input'
        concat tag :input, name: name, type: :hidden, value: false
        concat check_box_tag(name, value, checked, options)
      end
    end

    def select_input_tag(name, option_tags=nil, options={}, &block)
      if option_tags.is_a? Hash
        options = option_tags
        option_tags = nil
      end

      options[:label] ||= options.delete(:placeholder)

      option_tags ||= capture(&block).html_safe if block_given?
      
      input_tag(:select, name, option_tags.html_safe, options)
    end

    def switch_input_tag(name, checked = false, options = {})

      if checked.is_a? Hash
        options = checked
        checked = false
      end

      if label_text = options.delete(:label)
        label_text = content_tag(:span, class: 'label-text') { label_text }
      end
      
      if label_description = options.delete(:description)
        label_description = content_tag(:span, class: 'label-description') { label_description }
      end


      content_tag(:label, class: "check-switch switch-label #{ 'check-switch-full-width' if options[:full_width] }", data: { input: 'checkbox' }) do
        concat tag :input, name: name, type: :hidden, value: false
        concat label_text
        concat check_box_tag(name, true, checked, options)

        concat content_tag(:span, class: 'check-switch-panel') {
          concat content_tag(:span, class: 'check-switch-tick') { '' }
        }

        concat content_tag(:span, class: 'check-switch-label') { 'Enable' }
        concat label_description

      end

    end

    private

    def base_tag(name, value, options, type, &block)
      case type
      when :range
        range_input_tag(name, options)
      when :select
        value = value.html_safe if value
        select_tag(name, value, options)
      when :textarea
        value ||= capture(&block).html_safe if block_given?
        options[:class] = "#{options[:class]} empty".strip if value.nil?
        text_area_tag(name, value, options)
      else
        options[:class] = "#{options[:class]} empty".strip if value.nil?
        text_field_tag(name, value, options)
      end
    end

    def input_tag(type, name, value, options=nil, &block)
      type.to_sym! if type.is_a?( String ) 
      input_wrapper_options = { class: 'input-wrapper' }

      if value.is_a? Hash
        options = value
        value = nil
      end

      options[:autocomplete] ||= 'off'

      options[:data] ||= {}
      options[:data][:message] ||= options.delete(:message)
      options[:data]['unique-id'] = "input-#{SecureRandom.hex(8)}"
      options[:data][:default] ||= options.delete(:default)

      if errors = options.delete(:errors)
        options[:data]['invalid-value'] = value
      end

      if unit = options.delete(:unit)
        options[:data][:unit] = unit
      end

      if options.delete(:copy)
        copy = copy_button("[data-unique-id='#{options[:data]['unique-id']}']", tabindex: '-1')
      end

      if step_labels = options.delete(:step_labels) || options.delete(:external_labels)
        if step_labels.is_a?(Hash)
          step_labels.each do |label, value|
            l = dasherize(label.to_s.downcase)
            options[:data]['step-label-'+l] = value.join(';')
          end
        end
      end

      unit_label = options.delete(:unit_label)

      options = (INPUT_OPTIONS[type]||{}).deep_merge options
      options[:type] ||= type

      label_options = {
        class: "#{label_class(options[:type])} #{options.delete(:class)} #{'remotely-invalid' if errors} #{'copy-input' if copy}",
        data: { 'input-type' => type }
      }

      if options[:type] != :textarea && placeholder = options[:placeholder]
        placeholder = content_tag(:span, class: 'placeholder-label-text') { placeholder }
        label_options[:class] += ' placeholder-label'
      end

      if label_description = options.delete(:description)
        options['aria-describedby'] = options[:data]['unique-id'] + '-description'
        label_description = content_tag(:span, id: options['aria-describedby'], class: 'label-description') { label_description }
      end

      if options[:label] && label_text = options.delete(:label)
        options['aria-labelledby'] = options[:data]['unique-id'] + '-label'

        label_text = content_tag(:span, class: 'label-text') {
          concat content_tag(:span, id: options['aria-labelledby']){ label_text }
          if options[:data][:default]
            concat " "
            concat restore_default(options[:data]['unique-id']) 
          end
        }
        if self && label_description
          label_options[:class] += ' description-label'
        else
          label_options[:class] += ' standard-label'
        end
      end

      options['aria-label'] = options[:placeholder] if placeholder && !label_text
      
      if requires_restart = options.delete(:requires_restart)
        label_options[:class] += ' requires-restart'
        describe_id = options[:data]['unique-id'] + '-restart-description'
        options[:data][:diff_class] = 'requires-restart'
        options[:data][:diff_note] = "<span class='has-tooltip' aria-label='Requires restart'>#{use_svg("status-failed")}</span>"
        (options['aria-describedby'] ||= '') << " #{describe_id}"
        requires_restart = content_tag(:span, id: describe_id, class: 'requires-restart-input-message') {
          concat "#{use_svg("status-failed", class: 'requires-restart-icon')} ".html_safe
          if requires_restart.is_a?(String)
            concat requires_restart
          else
            concat "Changing this requires a database restart."
          end
        }
      end

      options[:class] = 'input'

      if options.delete(:shell_text)
        input_wrapper_options[:class] += ' shell-text'
      end

      tag = base_tag(name, value, options, type, &block)

      label = content_tag(:label, label_options) {
        concat label_text
        concat label_description

        if type == :select
          concat tag
          concat placeholder
        else
          concat content_tag(:span, input_wrapper_options) {
            concat tag
            concat placeholder
            concat copy if copy
            concat unit_label_html(unit_label) if unit_label
          }
        end

        if errors
          concat content_tag(:span, class: "remote-validation-message") {
            content_tag(:span, class: "validation-message-text", role:"alert") {
              if errors.size > 1
                concat content_tag(:span, class: 'validation-error-notice') { 'There were some errors:' }
                concat " "
              end
              errors.each do |error|
                concat content_tag(:span, class: 'valdiation-error-item') {
                  concat error
                }
                concat " "
              end
            }
          }.html_safe

        end
        concat requires_restart if requires_restart
      }

      label
    end

    def unit_label_html(label)
      if label
        content_tag(:span, class: 'input-unit') {
          %Q{<span class="input-unit-name">#{label}</span>}.html_safe
        }
      end
    end

    private

    def restore_default(selector)
      content_tag(:a, href: "#", tabindex: '-1', class: "restore-default", data: { 'restore-default' => "[data-unique-id='#{selector}']" }) {
        content_tag(:span, "aria-label" => "Restore default value", class: "has-tooltip" ) { "⎋" }
      }
    end

    def label_class( type )
      type = case type
      when :tel, :password, :number, :url, :email, :search
        "text"
      when :checkbox, :radio
        "tick"
      else
        type.to_s
      end

      "#{type}-label"
    end

    def tick_wrapper( name, options, &block )

      tag = capture(&block).html_safe

      options.delete(:required)

      tick = content_tag(:span, class: 'tick') {''}

      classnames = ['label-text']
      classnames << 'hidden' if options.delete(:hidden_label)
      label = content_tag(:span, class: classnames.join(' ')) { options.delete(:label) || name }

      options[:class] ||= ' '
      options[:class] << "#{label_class( options.delete(:type) )} tick-box"

      content_tag(:label, options ) {
        concat tag.html_safe
        concat tick
        concat content_tag(:span, class: "label-text-wrapper") { label }
      }
    end

    def range_input_tag(name, options={})
      options = options.stringify_keys
      classnames = options.delete('class') || ''

      data = options['data'] || {}
      data['input'] ||= name

      if options['position_label']
        data['position_label'] = options['position_label']
      end

      # Set values (and max based on values size)
      if values = options['values']
        data['values'] = values.join(',')
        options['max'] ||= values.size - 1
      end

      # Support legacy option
      options['labels'] ||= options['label']

      if labels = options['labels']
        if labels.is_a?(Array)
          data['label'] = labels.join(';')
          options['max'] ||= labels.size - 1
        elsif labels.is_a?(Hash)
          labels.each do |label, value|
            data['label-'+dasherize(label.to_s.downcase)] = value.join(';')
            options['max'] ||= value.size - 1
          end
        end
      end

      if labels == false
        data['label'] = 'false'
      end

      if step_labels = options.delete(:step_labels) || options.delete(:external_labels)
        if step_labels.is_a?(Hash)
          step_labels.each do |label, value|
            l = dasherize(label.to_s.downcase)
            data['step-label-'+l] = value.join(';')
          end
        end
      end

      if before = options['before']
        if before.is_a?(String)
          data['before-label'] = before
        else
          before.each do |key, value|
            data["before-label-#{key}"] = value
          end
        end
      end

      if mark = options['mark']
        data['mark'] = mark.join(',')
      end

      if after = options['after']
        if after.is_a?(String)
          data['after-label'] = after
        else
          after.each do |key, value|
            data["after-label-#{key}"] = value
          end
        end
      end

      if options['show_segments'] == true
        classnames << ' slider-show-segments'
      end   

      if line_labels = options['line_labels']
        data['line_labels'] = []
        line_labels.each do |k, v|
          data['line_labels'] << "#{k}:#{v}"
        end
        data['line_labels'] = data['line_labels'].join(';')
      end

      options['value'] ||= options['min'] || 0

      if data[:default] && options['values']
        data[:default] = options['values'].find_index(data[:default]) || data[:default]
      end

      html_options = {
        id:                  options.delete('id'),
        class:               classnames, 
        type:                "range", 
        min:                 options['min'], 
        max:                 options['max'],
        step:                options['step'],
        value:               options['value'],
        "aria-label"      => options['aria-label'],
        "aria-labelledby" => options['aria-labelledby'],
        "aria-describedby" => options['aria-describedby']
      }.update({ data: data })

      tag :input, html_options
    end

    def dasherize(input)
      input.gsub(/[\W,_]/, '-').gsub(/-{2,}/, '-')
    end

  end
end
